@@ -1,6 +1,6 @@
 /*
-   Copyright 2011-2013 ALT Linux
-   Copyright 2011-2013 Michael Pozhidaev
+   Copyright 2011-2014 ALT Linux
+   Copyright 2011-2014 Michael Pozhidaev
 
    This file is part of the Deepsolver.
 
@@ -19,7 +19,8 @@
 #include"deepsolver/PkgSnapshot.h"
 #include"deepsolver/InlineIO.h"
 
-#define THROW_INTERNAL_ERROR throw Deepsolver::OperationException(Deepsolver::OperationException::InternalIOProblem)
+//#define THROW_INTERNAL_ERROR throw Deepsolver::OperationException(Deepsolver::OperationException::InternalIOProblem)
+#define THROW_INTERNAL_ERROR //FIXME:
 
 DEEPSOLVER_BEGIN_NAMESPACE
 DEEPSOLVER_BEGIN_PKG_SNAPSHOT_NAMESPACE
@@ -171,7 +172,7 @@ void enhance(Snapshot& snapshot,
 	     int flags,
 	     ConstCharVector& strings)
 {
-  logMsg(LOG_DEBUG, "enhancing:starting enhancing procedure with %zu new packages", enhanceWith.size());
+  logMsg(LOG_DEBUG, "snapshot:starting enhancing procedure with %zu new packages", enhanceWith.size());
   const clock_t started = clock();
   if (enhanceWith.empty())
     return;
@@ -423,7 +424,7 @@ void loadFromFile(Snapshot& snapshot,
 		  ConstCharVector& strings)
 {
   assert(!fileName.empty());
-  logMsg(LOG_INFO, "snapshot:starting reading from binary file \'%s\'", fileName.c_str());
+  logMsg(LOG_DEBUG, "snapshot:starting reading from binary file \'%s\'", fileName.c_str());
   assert(snapshot.pkgNames.empty());
   assert(snapshot.pkgs.empty());
   assert(snapshot.relations.empty());
@@ -691,13 +692,17 @@ void removeEqualPkgs(Snapshot& snapshot)
   logMsg(LOG_DEBUG, "snapshot:%zu doubled packages are filtered out", offset);
 }
 
-void printContent(const Snapshot& snapshot, std::ostream& s)
+void printContent(const Snapshot& snapshot, 
+		  bool withIds,
+		  std::ostream& s)
 {
   for(PkgVector::size_type i = 0;i < snapshot.pkgs.size();i++)
     {
       const PkgSnapshot::Pkg& p = snapshot.pkgs[i]; 
       assert(p.pkgId != BadPkgId);
-      s << ((p.flags & PkgFlagInstalled)?"Installed package: ":"Package: ");
+      if (withIds)
+	s << "#" << i << ((p.flags & PkgFlagInstalled)?", installed: ": ": "); else
+	s << ((p.flags & PkgFlagInstalled)?"Installed package: ":"Package: ");
       s << pkgIdToStr(snapshot, p.pkgId) << "-";
       if (p.epoch > 0)
 	s << p.epoch << ":";
