@@ -1,6 +1,6 @@
 /*
-   Copyright 2011-2013 ALT Linux
-   Copyright 2011-2013 Michael Pozhidaev
+   Copyright 2011-2014 ALT Linux
+   Copyright 2011-2014 Michael Pozhidaev
 
    This file is part of the Deepsolver.
 
@@ -28,8 +28,7 @@
 #include"deepsolver/StringUtils.h"
 #include"deepsolver/types.h"
 #include"deepsolver/Pkg.h"
-#include"deepsolver/doubling.h"
-#include"deepsolver/Dichotomy.h"
+#include"deepsolver/utils.h"
 
 #define DEEPSOLVER_BEGIN_NAMESPACE namespace Deepsolver {
 #define DEEPSOLVER_END_NAMESPACE }
