@@ -1,6 +1,6 @@
 /*
-   Copyright 2011-2013 ALT Linux
-   Copyright 2011-2013 Michael Pozhidaev
+   Copyright 2011-2014 ALT Linux
+   Copyright 2011-2014 Michael Pozhidaev
 
    This file is part of the Deepsolver.
 
@@ -18,7 +18,7 @@
 #ifndef DEEPSOLVER_ALWAYS_TRUE_CONTINUE_REQUEST_H
 #define DEEPSOLVER_ALWAYS_TRUE_CONTINUE_REQUEST_H
 
-#include"deepsolver/AbstractOperationContinueRequest.h"
+#include"deepsolver/AbstractContinueRequest.h"
 
 namespace Deepsolver
 {
