@@ -1,6 +1,6 @@
 /*
-   Copyright 2011-2013 ALT Linux
-   Copyright 2011-2013 Michael Pozhidaev
+   Copyright 2011-2014 ALT Linux
+   Copyright 2011-2014 Michael Pozhidaev
 
    This file is part of the Deepsolver.
 
@@ -17,13 +17,13 @@
 
 #include"deepsolver/deepsolver.h"
 #include"deepsolver/OperationCore.h"
+#include"deepsolver/ExceptionMessagesEn.h"
 #include"Messages.h"
 
 using namespace Deepsolver;
 
 int main(int argc, char* argv[])
 {
-  messagesProgramName = "ds-conf";
   setlocale(LC_ALL, "");
   initLogging(LOG_DEBUG, 0);
   try{
@@ -33,24 +33,11 @@ int main(int argc, char* argv[])
     conf.commit();
     conf.printConfigData(std::cout);
   }
-  catch (const ConfigFileException& e)
+  catch(const AbstractException& e)
     {
-      Messages(std::cerr).onConfigSyntaxError(e);
-      return EXIT_FAILURE;
-    }
-  catch (const ConfigException& e)
-    {
-      Messages(std::cerr).onConfigError(e);
-      return EXIT_FAILURE;
-    }
-  catch(const SystemException& e)
-    {
-      Messages(std::cerr).onSystemError(e);
-      return EXIT_FAILURE;
-    }
-  catch(const NotImplementedException& e)
-    {
-      std::cerr << "Feature not implemented:" << e.getMessage() << std::endl;
+      ExceptionMessagesEn messages;
+      e.accept(messages);
+      std::cerr << messages.getMsg();
       return EXIT_FAILURE;
     }
   return EXIT_SUCCESS;
