@@ -1,6 +1,6 @@
 /*
-   Copyright 2011-2013 ALT Linux
-   Copyright 2011-2013 Michael Pozhidaev
+   Copyright 2011-2014 ALT Linux
+   Copyright 2011-2014 Michael Pozhidaev
 
    This file is part of the Deepsolver.
 
@@ -17,7 +17,7 @@
 
 #include"deepsolver/deepsolver.h"
 #include"deepsolver/IndexCore.h"
-#include"CliParser.h"
+#include"deepsolver/CliParser.h"
 
 #define PREFIX "ds-provides:"
 
@@ -136,19 +136,10 @@ void parseCmdLine(int argc, char* argv[])
   }
   catch (const CliParserException& e)
     {
-      switch (e.getCode())
-	{
-	case CliParserException::NoPrgName:
-	  std::cerr << PREFIX << "Command line has no program name" << std::endl;
-	  exit(EXIT_FAILURE);
-	case CliParserException::MissedArgument:
-	  std::cerr << PREFIX << "Command line argument \'" << e.getArg() << "\' requires additional parameter" << std::endl;exit(EXIT_FAILURE);
-	  exit(EXIT_FAILURE);
-	default:
-	  assert(0);
-	} //switch();
+      std::cerr << "command line error:" << e.getMessage() << std::endl;
+      exit(EXIT_FAILURE);
     }
-  if (cliParser.wasKeyUsed("--help"))
+  if (cliParser.isKeyUsed("--help"))
     {
       printHelp();
 	exit(EXIT_SUCCESS);
@@ -162,7 +153,7 @@ void parseCmdLine(int argc, char* argv[])
     }
   params.indexPath = cliParser.files[0];
   std::string arg;
-  if (cliParser.wasKeyUsed("--ref-sources", arg))
+  if (cliParser.isKeyUsed("--ref-sources", arg))
     splitColonDelimitedList(arg, params.providesRefsSources);
 }
 
@@ -171,14 +162,14 @@ int main(int argc, char* argv[])
   setlocale(LC_ALL, "");
   initCliParser();
   parseCmdLine(argc, argv);
-  initLogging(cliParser.wasKeyUsed("--debug")?LOG_DEBUG:LOG_INFO, cliParser.wasKeyUsed("--log") && !cliParser.wasKeyUsed("--quiet"));
+  initLogging(cliParser.isKeyUsed("--debug")?LOG_DEBUG:LOG_INFO, cliParser.isKeyUsed("--log") && !cliParser.isKeyUsed("--quiet"));
   try {
-    if (!cliParser.wasKeyUsed("--log") && !cliParser.wasKeyUsed("--quiet"))
+    if (!cliParser.isKeyUsed("--log") && !cliParser.isKeyUsed("--quiet"))
       printLogo();
     params.readInfoFile(Directory::mixNameComponents(params.indexPath, REPO_INDEX_INFO_FILE));
 
     std::string arg;
-    if (cliParser.wasKeyUsed("--external-provides", arg))
+    if (cliParser.isKeyUsed("--external-provides", arg))
       {
 	File f;
 	f.openReadOnly(arg);
@@ -193,21 +184,21 @@ int main(int argc, char* argv[])
 	  }
       }
 
-    IndexReconstructionListener listener(cliParser.wasKeyUsed("--log") || cliParser.wasKeyUsed("--quiet"));
+    IndexReconstructionListener listener(cliParser.isKeyUsed("--log") || cliParser.isKeyUsed("--quiet"));
     IndexCore indexCore(listener);
     indexCore.refilterProvides(params);
   }
-  catch(const DeepsolverException& e)
+  catch(const AbstractException& e)
     {
       logMsg(LOG_CRIT, "%s error:%s", e.getType().c_str(), e.getMessage().c_str());
-      if (!cliParser.wasKeyUsed("--log"))
+      if (!cliParser.isKeyUsed("--log"))
 	std::cerr << "ERROR:" << e.getMessage() << std::endl;
       return EXIT_FAILURE;
     }
   catch(std::bad_alloc)
     {
       logMsg(LOG_CRIT, "No enough memory");
-      if (!cliParser.wasKeyUsed("--log"))
+      if (!cliParser.isKeyUsed("--log"))
 	std::cerr << "ERROR:No enough memory" << std::endl;
 	  return EXIT_FAILURE;
     }
