@@ -1,6 +1,6 @@
 /*
-   Copyright 2011-2013 ALT Linux
-   Copyright 2011-2013 Michael Pozhidaev
+   Copyright 2011-2014 ALT Linux
+   Copyright 2011-2014 Michael Pozhidaev
 
    This file is part of the Deepsolver.
 
@@ -17,10 +17,11 @@
 
 #include"deepsolver/deepsolver.h"
 #include"deepsolver/OperationCore.h"
+#include"deepsolver/ExceptionMessagesEn.h"
 #include"TransactionProgress.h"
 #include"Messages.h"
-#include"CliParser.h"
-#include"PackageListPrinting.h"
+#include"deepsolver/CliParser.h"
+#include"PkgListPrinting.h"
 #include"AlwaysTrueContinueRequest.h"
 #include"FilesFetchProgress.h"
 
@@ -120,123 +121,86 @@ void parseCmdLine(int argc, char* argv[])
   }
   catch (const CliParserException& e)
     {
-      switch (e.getCode())
-	{
-	case CliParserException::NoPrgName:
-	  Messages(std::cerr).onMissedProgramName();
-	  exit(EXIT_FAILURE);
-	case CliParserException::MissedArgument:
-	  Messages(std::cout).onMissedCommandLineArgument(e.getArg());
-	  exit(EXIT_FAILURE);
-	default:
-	  assert(0);
-	} //switch();
+      ExceptionMessagesEn messages;
+      e.accept(messages);
+      exit(EXIT_FAILURE);
     }
-  if (cliParser.wasKeyUsed("--help"))
+  if (cliParser.isKeyUsed("--help"))
     {
       Messages(std::cout).dsRemoveHelp(cliParser);
       exit(EXIT_SUCCESS);
     }
 }
 
-int main(int argc, char* argv[])
+int doMainWork()
 {
-  messagesProgramName = "ds-remove";
-  setlocale(LC_ALL, "");
-  parseCmdLine(argc, argv);
-  initLogging(cliParser.wasKeyUsed("--debug")?LOG_DEBUG:LOG_INFO, cliParser.wasKeyUsed("--log"));
-  try{
-    TransactionProgress transactionProgress(std::cout, cliParser.wasKeyUsed("--log"));
-    ConfigCenter conf;
-    conf.loadFromFile(DEFAULT_CONFIG_FILE_NAME);
-    conf.loadFromDir(DEFAULT_CONFIG_DIR_NAME);
-    conf.commit();
-    if (!cliParser.wasKeyUsed("--log"))
-      Messages(std::cout).dsRemoveLogo();
-    OperationCore core(conf);
-    UserTask userTask;
-    for(StringVector::size_type i = 0;i < cliParser.files.size();i++)
-      if (!cliParser.files[i].empty())
-	userTask.namesToRemove.insert(cliParser.files[i]);
-    if (userTask.namesToRemove.empty())
-      {
-	Messages(std::cerr).onNoPackagesMentionedError();
-	return EXIT_FAILURE;
-      }
-    if (!cliParser.wasKeyUsed("--sat"))
-      {
-	std::auto_ptr<TransactionIterator> it = core.transaction(transactionProgress, userTask);
-	if (cliParser.wasKeyUsed("--urls"))
-	  {
-	    printUrls(*it.get());
-	    return EXIT_SUCCESS;
-	  }
-	if (!cliParser.wasKeyUsed("--files"))
-	  {
-	    PackageListPrinting(conf).printSolution(*it.get());
-	    std::cout << std::endl;
-	  }
-	if (it->emptyTask() || cliParser.wasKeyUsed("--dry-run"))
+  TransactionProgress transactionProgress(std::cout, cliParser.isKeyUsed("--log"));
+  ConfigCenter conf;
+  conf.loadFromFile(DEFAULT_CONFIG_FILE_NAME);
+  conf.loadFromDir(DEFAULT_CONFIG_DIR_NAME);
+  conf.commit();
+  if (!cliParser.isKeyUsed("--log"))
+    Messages(std::cout).dsRemoveLogo();
+  OperationCore core(conf);
+  UserTask userTask;
+  for(StringVector::size_type i = 0;i < cliParser.files.size();i++)
+    if (!cliParser.files[i].empty())
+      userTask.namesToRemove.insert(cliParser.files[i]);
+  if (userTask.namesToRemove.empty())
+    {
+      Messages(std::cerr).onNoPkgMentionedError();
+      return EXIT_FAILURE;
+    }
+  if (!cliParser.isKeyUsed("--sat"))
+    {
+      TransactionIterator::Ptr it = core.transaction(transactionProgress, userTask);
+      if (cliParser.isKeyUsed("--urls"))
+	{
+	  printUrls(*it.get());
 	  return EXIT_SUCCESS;
-	if (!cliParser.wasKeyUsed("--files"))
-	  if (!Messages(std::cout).confirmContinuing())
-	    return 0;
-	std::cout << std::endl;
-	FilesFetchProgress progress(std::cout, cliParser.wasKeyUsed("--log"));
-	AlwaysTrueContinueRequest continueRequest;
-	it->fetchPackages(progress, continueRequest);
-	if (cliParser.wasKeyUsed("--files"))
-	  {
-	    printFiles(*it.get());
-	    return EXIT_SUCCESS;
-	  }
-	it->makeChanges();
-      } else
-      {
-	const std::string res = core.generateSat(transactionProgress, userTask);
-	std::cout << std::endl;
-	std::cout << res;
-      }
-  }
-  catch (const ConfigFileException& e)
+	}
+      if (!cliParser.isKeyUsed("--files"))
+	PkgListPrinting(conf).printSolution(*it.get(), cliParser.isKeyUsed("--log"));
+      if (it->emptyTask() || cliParser.isKeyUsed("--dry-run"))
+	return EXIT_SUCCESS;
+      if (!cliParser.isKeyUsed("--files"))
+	if (!Messages(std::cout).confirmContinuing())
+	  return 0;
+      std::cout << std::endl;
+      FilesFetchProgress progress(std::cout, cliParser.isKeyUsed("--log"));
+      AlwaysTrueContinueRequest continueRequest;
+      it->fetchPackages(progress, continueRequest);
+      if (cliParser.isKeyUsed("--files"))
+	{
+	  printFiles(*it.get());
+	  return EXIT_SUCCESS;
+	}
+      it->makeChanges();
+    } else
     {
-      Messages(std::cerr).onConfigSyntaxError(e);
-      return EXIT_FAILURE;
-    }
-  catch (const ConfigException& e)
-    {
-      Messages(std::cerr).onConfigError(e);
-      return EXIT_FAILURE;
-    }
-  catch(const TaskException& e)
-    {
-       Messages(std::cerr).onTaskError(e);
-       return EXIT_FAILURE;
-    }
-  catch(const OperationException& e)
-    {
-      Messages(std::cerr).onOperationError(e);
-      return EXIT_FAILURE;
-    }
-  catch(const CurlException& e)
-    {
-      Messages(std::cerr).onCurlError(e);
-      return EXIT_FAILURE;
-    }
-  catch(const PackageBackEndException& e)
-    {
-      Messages(std::cerr).onPackageBackEndError(e);
-      return EXIT_FAILURE;
-    }
-  catch(const SystemException& e)
-    {
-      Messages(std::cerr).onSystemError(e);
-      return EXIT_FAILURE;
-    }
-  catch(const NotImplementedException& e)
-    {
-      std::cerr << "Feature not implemented:" << e.getMessage() << std::endl;
-      return EXIT_FAILURE;
+      std::cout << std::endl;
+      core.generateSat(transactionProgress, userTask, std::cout);
     }
   return EXIT_SUCCESS;
 }
+
+int main(int argc, char* argv[])
+{
+  setlocale(LC_ALL, "");
+  parseCmdLine(argc, argv);
+  initLogging(cliParser.isKeyUsed("--debug")?LOG_DEBUG:LOG_INFO, cliParser.isKeyUsed("--log"));
+  try{
+    return doMainWork();
+  }
+  catch(const AbstractException& e)
+    {
+      if (!cliParser.isKeyUsed("--log"))
+	{
+	  ExceptionMessagesEn messages;
+	  e.accept(messages);
+	  std::cerr << messages.getMsg();
+	} else
+	std::cerr << e.getType() << " error:" << e.getMessage() << std::endl;
+      return EXIT_FAILURE;
+    }
+}
